#!/bin/bash
#
# bash_validator:bin/bash_validator-add-to-ignore.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/bash_validator-config-api.sh" || exit $?

function usage {
  cat <<EOF
Usage

  $( basename "$0" ) --help
  $( basename "$0" ) [--root-directory DIR] [--user|--local] FILE1 [FILE2]...

Options

  -h,       --help      Display help message and exit.
  -U,       --user      Apply configuration at user level.
  -L,       --local     Apply configuration at local level.
  -d DIR,   --root-directory DIR
                        Define local level directory
                        (if ommited use current directory)

EOF
}

function normalize_config {
  local -r config_filename="${1}"
  local -r config_safe="${config_filename}.safe"

  echoinfo "Normalize: '${config_filename}'"
  LC_ALL=C sort --unique <"${config_filename}" >"${config_safe}" || return $?
  mv -f "${config_safe}" "${config_filename}" || return $?
}

function add_to_ignore_user_config {
  local -r filename="$1"

  local full_filename=
  full_filename="$( realpath "${filename}" )" || return $?

  echoinfo "Adding: '${full_filename}'"
  mkdir -vp "${USER_CONFIG_HOME}" || return $?
  echo "${full_filename}" >>"${USER_IGNORE_FILE}" || return 5 # write error
  normalize_config "${USER_IGNORE_FILE}" || return $?
}

function add_to_ignore_local_config {
  local -r filename="${1}"

  local local_filename=
  local_filename="$( realpath --relative-to="${param_root_directory}" "${filename}" )" || return $?
  echoinfo "Adding: '${local_filename}'"

  echo "${local_filename}" >>"${param_root_directory}/${LOCAL_IGNORE_FILENAME}" || return 5 # write error
  normalize_config "${param_root_directory}/${LOCAL_IGNORE_FILENAME}" || return $?
}

function add_to_ignore {
  local -r filename="${1}"

  if [ -z "${filename}" ] ; then
    echoerror 'File name is required'
    return 1
  fi

  if [ ! -f "${filename}" ] ; then
    echoerror "Can't find file: '${filename}'"
    return 1
  fi

  case "${param_configuration_level}" in
    '--user')
      add_to_ignore_user_config "${filename}" || return $?
      ;;

    '--local')
      add_to_ignore_local_config "${filename}" || return $?
      ;;

    *)
      echofatal "Internal error: param_configuration_level='${param_configuration_level}'"
      return 1
  esac
}

function main {
  local    param=
  local    param_configuration_level=
  local -a param_files=()
  local    param_root_directory=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-U'|'--user')
        if [ -n "${param_configuration_level}" ] ; then
          echofatal "Parameters --user and --local can't be define twice."
          return 1
        fi
        param_configuration_level='--user'
        ;;

      '-L'|'--local')
        if [ -n "${param_configuration_level}" ] ; then
          echofatal "Parameters --user and --local can't be define twice."
          return 1
        fi
        param_configuration_level='--local'
        ;;

      '-d'|'--root-directory')
        if [ -n "${param_root_directory}" ] ; then
          echofatal "Parameter --root-directory can't be define twice."
          return 1
        fi
        param_root_directory="$( P "$@" )" && shift
        ;;

       *)
         param_files+=( "${param}" )
         ;;
    esac
  done

  if [ "${#param_files[@]}" -eq 0 ] ; then
    echo '*** ERROR: At least one file name is required'
    return 1
  fi

  if [ -z "${param_configuration_level}" ] ; then
    param_configuration_level='--user'
  fi

  if [ -n "${param_root_directory}" ] ; then
    if [ ! -d "${param_root_directory}" ] ; then
      echofatal "Can't find directory: '${param_root_directory}'"
      return 1
    fi
  else
    param_root_directory="$( pwd )"
  fi

  for param in "${param_files[@]}" ; do
    add_to_ignore "${param}" || return $?
  done
}

main "$@" || exit $?
