#!/bin/bash
#
# bash_validator:bin/bash_validator.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/bash_validator-config-api.sh" || exit $?

declare -g SHELLCHECK_ERROR='false'
declare -g VERBOSE='false'
declare -g VVERBOSE='false'
declare -g IGNORE_FILE="${USER_IGNORE_FILE}"

function display_usage {
  cat <<EOF
Usage: $0

  $( basename "$0" ) [--help]
  $( basename "$0" ) --sh-only [-v|-vv|--verbose] [-d|--root-directory ROOT_PATH]
  $( basename "$0" ) [-v|-vv|--verbose] [-d|--root-directory ROOT_PATH]

  -h,       --help      Display help message and exit.
            --sh-only   Check only *.sh files
  -v,       --verbose   Enable verbose mode
            -vv         Enable very verbose mode
  -d DIR,   --root-directory DIR
                        Start directory and recurse in it.

EOF
}

function echo_verbose {
  if [ "${VERBOSE}" = "true" ]; then
    cat  >&2 <<EOF
$@
EOF
  fi
}

function echo_vverbose {
  if [ "${VVERBOSE}" = "true" ]; then
    cat  >&2 <<EOF
$@
EOF
  fi
}

function do_shellcheck {
  local -r file="$1"
  local -r shell="${2:-}"

  local -a options=()

  if [ -f "${IGNORE_FILE}" ] ; then
    if grep --quiet "${file}" "${IGNORE_FILE}"; then
      echo_vverbose "Ignore ${file}"
      return 0
    fi

    local rpfile=
    rpfile="$( realpath "${file}" )"

    if grep --quiet "${rpfile}" "${IGNORE_FILE}"; then
      echo_vverbose "Ignore ${rpfile}"
      return 0
    fi
  fi

  if [ -n "${shell}" ] ; then
    options+=("--shell=${shell}")
  fi

  shellcheck "${options[@]}" "${file}" || SHELLCHECK_ERROR=true
}

function do_shellcheck_for_sh {
  local -r path="$1"

  local file=
  while IFS= read -r -d $'\0' file; do
    echo_verbose "${file}"
    do_shellcheck "${file}"
  done < <( find "${path}" -type f -not -path '*\/.git\/*' -not -path '*\/.metadata\/*' -name "*.sh" -print0 | sort -z )
}

function do_shellcheck_for_all {
  local -r path="$1"

  while IFS= read -r -d $'\0' file; do
    if [ -n "${file}" ] ; then
      do_shellcheck_for_all_check_entry "${file}" || return $?
    fi
  done < <( find "${path}" -type f -not -path '*\/.git\/*' -not -path '*\/.metadata\/*' -print0 | sort -z )
}

function do_shellcheck_for_all_check_entry {
  local -r file="$1"

  local magic=
  read -r -n 3 magic <"${file}"

  if [ "${magic}" = '#!/' ] ; then
    local header=
    read -r header < "${file}"
    header+=' '

    if [ '#!/bin/bash ' = "${header:0:12}" ] ; then
      #  '123456789012'
      echo_verbose "${file}:bash"

      do_shellcheck "${file}" 'bash'
    elif [ '#!/bin/sh ' = "${header:0:10}" ] ; then
      #    '1234567890'
      echo_verbose "${file}:sh"

      do_shellcheck "${file}" 'sh'
    elif [ '#!/bin/ksh ' = "${header:0:11}" ] ; then
      #    '12345678901'
      echo_verbose "${file}:ksh"

      do_shellcheck "${file}" 'ksh'
    elif [ '#!/usr/bin/env ' = "${header:0:15}" ] ; then
      #    '123456789012345'
      if [ -z "${header##* bash *}" ] ;then
        echo_verbose "${file}:bash"

        do_shellcheck "${file}" 'bash'
      elif [ -z "${header##* sh *}" ] ;then
        echo_verbose "${file}:sh"

        do_shellcheck "${file}" 'sh'
      elif [ -z "${header##* ksh *}" ] ;then
        echo_verbose "${file}:ksh"

        do_shellcheck "${file}" 'ksh'
      else
        echo_vverbose "*** ${file} IGNORE '${header}' (#!/usr/bin/env)" >&2
      fi
    else
      echo_vverbose "*** ${file} IGNORE '${header}' (#!/)" >&2
    fi
  elif [ "${file}" = "${file%.sh}.sh" ] ; then
    echo_verbose "${file}:end-with-sh"

    do_shellcheck "${file}"
  fi
}

function main {
  local param_function='all'
  local param_root_path='.'
  local param=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        display_usage
        return 0
        ;;

      '--sh-only')
        param_function='sh_only'
        ;;

      '-v'|'--verbose')
        VERBOSE='true'
        echo_verbose '# Verbose mode'
        ;;

      '-d'|'--root-directory')
        param_root_path="$( P "$@" )" && shift
        ;;

      '-vv')
        VERBOSE='true'
        VVERBOSE='true'
        echo_verbose '# Verbose mode'
        echo_vverbose '# vv mode'
        ;;

      *)
        echo "Unknown option: '${param}'" >&2
        display_usage >&2
        return 1
        ;;
    esac
  done

  if [ -f "${param_root_path}/${LOCAL_IGNORE_FILENAME}" ] ; then
    IGNORE_FILE="$( realpath "${param_root_path}/${LOCAL_IGNORE_FILENAME}" )" || return $?
  else
    echo_verbose "Local ignore file not found: '${LOCAL_IGNORE_FILENAME}'"
  fi

  echo_verbose "Use configuration based on: '${IGNORE_FILE}'"
  if [ ! -f "${IGNORE_FILE}" ] ; then
    echo_verbose "File not found: '${IGNORE_FILE}'"
  fi

  case "${param_function}" in
    'all')
      do_shellcheck_for_all "${param_root_path}"
      ;;

    'sh_only')
      do_shellcheck_for_sh "${param_root_path}"
      ;;

    *)
      display_usage >&2
      return 1
      ;;
  esac

  if [ "${SHELLCHECK_ERROR}" == 'true' ]; then
    echo_verbose "# status=SHELLCHECK_ERROR"
    exit 1
  else
    echo_verbose "# status=OK"
  fi
}

main "$@" || exit $?

