#!/bin/bash -e
#
# bash_validator:bin/install-shellcheck.sh
#
# Based on https://github.com/koalaman/shellcheck
#
declare -gr SHELLCHECK_LOCATION='/usr/bin/shellcheck'

if [ "$( type -t echofatal )" != 'function' ] ; then
  # shellcheck disable=SC1091
  source "$( dirname "$( dirname "${BASH_SOURCE[0]}" )" )/apis/bash-common-api/message-api.sh" || exit $?
fi

function get_arch {
  local arch=

  arch="$( uname -m )" || exit 1

  case "${arch}" in
    'i'?'86')
      echo 'i386'
      ;;

    'armv'*)
      echo 'armv6hf'
      ;;

    'x86_64'|'ppc64')
      echo "${arch}"
      ;;

    *)
      echo "${arch}"
      ;;
  esac
}

function install_shellcheck_from_github {
  local -r param_pkg_manager="$1"

  install_install_if_needed "${param_pkg_manager}" \
    xz-utils \
    wget

  local -r version='stable' # 'stable' or 'v0.4.7', or 'latest'
  local    arch=
  arch="$( get_arch )" || exit 1  # 'armv6hf' or 'x86_64'

  install_shellcheck_version_from_github "${version}" "${arch}"
}

function install_shellcheck_version_from_github {
  local -r version="$1" # 'stable' or 'v0.4.7', or 'latest'
  local -r arch="$2"    # 'armv6hf' or 'x86_64'

  if [ -z "${HOME:-}" ] ; then
    HOME="$( mktemp -d )" || exit 1
  fi

  mkdir -vp "${HOME}/.tmp" || exit 1
  local -r shellcheck_workspace="${HOME}/.tmp/shellcheck"

  if [ -d "${shellcheck_workspace}" ] ; then
    rm -fr "${shellcheck_workspace}" || exit 1
  fi
  mkdir -vp "${shellcheck_workspace}" || exit 1

  local -r url="https://github.com/koalaman/shellcheck/releases/download/${version?}/shellcheck-${version?}.linux.${arch}.tar.xz"

  echoinfo "Download: '${url}'" >&2
  pushd "${shellcheck_workspace}" >/dev/null || exit 1
  wget -qO- "${url}" | tar -xJv || exit 1
  popd  >/dev/null || exit 1

  local -r extracted_binary="${shellcheck_workspace}/shellcheck-${version}/shellcheck"
  if [ ! -f "${extracted_binary}" ] ; then
    echofatal "*** FATAL: Can't find '${extracted_binary}'" >&2
    exit 1
  fi
  chmod 0755 "${extracted_binary}" || exit 1
  "${extracted_binary}" -V || exit 1 # check if binary is valid.

  echo "Copy binary to '${SHELLCHECK_LOCATION}'" >&2 || exit 1
  cp -f "${extracted_binary}" "${SHELLCHECK_LOCATION}" || exit 1
  chown 'root':'root' "${SHELLCHECK_LOCATION}" || exit 1
  chmod 0755          "${SHELLCHECK_LOCATION}" || exit 1

  rm -frv "${shellcheck_workspace}" || exit 1
  rm -rv "${HOME}/.tmp" || return 0 # Remove only if empty.

  echoinfo "${SHELLCHECK_LOCATION} -V" >&2
  "${SHELLCHECK_LOCATION}" -V || exit 1

  echoinfo 'shellcheck -V' >&2
  shellcheck -V || exit 1

  local shellcheck_location=
  shellcheck_location="$( command -v shellcheck )"
  echoinfo "shellcheck location: '${shellcheck_location}'"

  if [ "${shellcheck_location}" = "${SHELLCHECK_LOCATION}" ] ; then
    echookay "shellcheck location is valid."
  else
    echoerror "shellcheck location is not valid. Expected '${SHELLCHECK_LOCATION}', found :'${shellcheck_location}'"
    cat <<EOF
Another version of shellcheck exist on you system.

EOF
  fi
}
